/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 * 
 * Author:
 *   Adam Lock <adamlock@netscape.com>
 *
 * Contributor(s): 
 */
#include "stdafx.h"
#include "IEHtmlDocument.h"
#include "IEHtmlElementCollection.h"
#include "IEHTMLElement.h"
#include "nsIDOMHTMLCollection.h"
#include "MozillaBrowser.h"
#include "nsHTMLTags.h"
#include "DummyBrowser.h"

#include <stack>

CIEHtmlDocument::CIEHtmlDocument() {
  m_pParent = NULL;
  m_pNative = nsnull;
  m_pDummy = NULL;
  CIEHtmlSelectionObjectInstance::CreateInstance(&m_pSelection);
  m_pSelection->AddRef();
  nsHTMLTags::AddRefTable();
}


CIEHtmlDocument::~CIEHtmlDocument() {
  m_pSelection->Release();
  nsHTMLTags::ReleaseTable();
  if (m_pDummy) m_pDummy->Release();
}


void CIEHtmlDocument::SetParent(CMozillaBrowser *parent)
{
        m_pParent = parent;
}


void CIEHtmlDocument::SetNative(nsIDOMHTMLDocument *native)
{
    m_pNative = native;
}


HRESULT CIEHtmlDocument::GetIDispatch(IDispatch **pDispatch)
{
        if (pDispatch == NULL)
        {
                return E_INVALIDARG;
        }

        return QueryInterface(IID_IDispatch,(void**)pDispatch);

/* Original code

        IDispatch *pDisp = (IDispatch *) this;
        NG_ASSERT(pDisp);
        pDisp->AddRef();
        *pDispatch = pDisp;

        return S_OK;
*/
}


///////////////////////////////////////////////////////////////////////////////
// IHTMLDocument methods

HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_Script(IDispatch __RPC_FAR *__RPC_FAR *p)
{
        return E_NOTIMPL;
}

///////////////////////////////////////////////////////////////////////////////
// IHTMLDocument2 methods

struct HtmlPos
{
        CIPtr(IHTMLElementCollection) m_cpCollection;
        long m_nPos;

        HtmlPos(IHTMLElementCollection *pCol, long nPos) :
                m_cpCollection(pCol),
                m_nPos(nPos)
        {
        }
};


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_all(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }

  *p = NULL;

  std::vector< CIPtr(IDispatch) > cNodeList;

  // Get all elements
  CIEHtmlElementCollectionInstance *pCollection = NULL;
  CIEHtmlElementCollection::CreateFromParentNode(this, (CIEHtmlElementCollection **) &pCollection, TRUE);
  if (pCollection) {
    pCollection->AddRef();
  }

  *p = pCollection;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_body(IHTMLElement __RPC_FAR *__RPC_FAR *p) {
  *p = NULL;
  return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_activeElement(IHTMLElement __RPC_FAR *__RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_images(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p)
{
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMHTMLCollection *pCollection = NULL;

  m_pNative->GetImages(&pCollection);

  CIEHtmlElementCollection::CreateFromDOMHTMLCollection(pCollection,
                                                       (CIEHtmlElementCollection **) &pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_applets(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p)
{
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMHTMLCollection *pCollection = NULL;

  m_pNative->GetApplets(&pCollection);

  CIEHtmlElementCollection::CreateFromDOMHTMLCollection(pCollection,
                                                       (CIEHtmlElementCollection **) &pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_links(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p)
{
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMHTMLCollection *pCollection = NULL;

  m_pNative->GetLinks(&pCollection);

  CIEHtmlElementCollection::CreateFromDOMHTMLCollection(pCollection,
                                                       (CIEHtmlElementCollection **) &pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_forms(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMHTMLCollection *pCollection = NULL;

  m_pNative->GetForms(&pCollection);

  CIEHtmlElementCollection::CreateFromDOMHTMLCollection(pCollection,
                                                       (CIEHtmlElementCollection **) &pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_anchors(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMHTMLCollection *pCollection = NULL;

  m_pNative->GetAnchors(&pCollection);

  CIEHtmlElementCollection::CreateFromDOMHTMLCollection(pCollection,
                                                       (CIEHtmlElementCollection **) &pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_title(BSTR v) {
  nsString value(OLE2W(v));
  if (m_pNative == NULL || m_pNative->SetTitle(value)) {
    return E_FAIL;
  }
  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_title(BSTR __RPC_FAR *p) {
  nsAutoString value;
  if (m_pNative == NULL || m_pNative->GetTitle(value)) {
    return E_FAIL;
  }
  *p = SysAllocString(value.GetUnicode()); 
  return p ? S_OK : E_OUTOFMEMORY;
}
//
// Helper function: Create the collection from node list
//
HRESULT SibCreateCollectionFromNodeList(nsIDOMNodeList *pNodeList,
                                        CIEHtmlElementCollection *pCollection) {
  if (!pCollection || !pNodeList) {
    return E_UNEXPECTED;
  }

  PRUint32 i, counter=0;
  nsIDOMNode *pIDOMNode;

  pNodeList->GetLength(&counter);
  for (i=0; i<counter; i++) {
    if (FAILED(pNodeList->Item(i,&pIDOMNode))) {
      return E_UNEXPECTED;
    }

    if (FAILED(pCollection->PopulateFromDOMNode(pIDOMNode,FALSE,L"ADD_THIS"))) {
      return E_OUTOFMEMORY;
    }
    pIDOMNode->Release();
  }

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_scripts(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMNodeList *pCollection = NULL;

  nsString szName(OLE2W(L"SCRIPT"));

  m_pNative->GetElementsByTagName(szName, &pCollection);

  CIEHtmlElementCollectionInstance::CreateInstance(&pCollection2);
  if (!pCollection2) {
    pCollection->Release();
    return E_OUTOFMEMORY;
  }
  SibCreateCollectionFromNodeList(pCollection, pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_designMode(BSTR v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_designMode(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_selection(IHTMLSelectionObject __RPC_FAR *__RPC_FAR *p) {
  if (!p) return E_INVALIDARG;
  m_pSelection->AddRef();
  *p=m_pSelection;
  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_readyState(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_frames(IHTMLFramesCollection2 __RPC_FAR *__RPC_FAR *p) {

  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMNodeList *pCollection = NULL;

  nsString szName(OLE2W(L"FRAME"));

  m_pNative->GetElementsByTagName(szName, &pCollection);

  CIEHtmlElementCollectionInstance::CreateInstance(&pCollection2);
  if (!pCollection2) {
    pCollection->Release();
    return E_OUTOFMEMORY;
  }
  SibCreateCollectionFromNodeList(pCollection, pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  pCollection2->QueryInterface(IID_IHTMLFramesCollection2, (void**)p);

  return S_OK;

}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_embeds(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMNodeList *pCollection = NULL;

  nsString szName(OLE2W(L"EMBED"));

  m_pNative->GetElementsByTagName(szName, &pCollection);

  CIEHtmlElementCollectionInstance::CreateInstance(&pCollection2);
  if (!pCollection2) {
    pCollection->Release();
    return E_OUTOFMEMORY;
  }
  SibCreateCollectionFromNodeList(pCollection, pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_plugins(IHTMLElementCollection __RPC_FAR *__RPC_FAR *p) {
  // Validate parameters
  if (p == NULL) {
    return E_INVALIDARG;
  }
  *p = NULL;
  if (!m_pNative) {
    return E_UNEXPECTED;
  }

  // Get elements
  CIEHtmlElementCollectionInstance *pCollection2 = NULL;
  nsIDOMNodeList *pCollection = NULL;

  nsString szName(OLE2W(L"PLUGIN"));

  m_pNative->GetElementsByTagName(szName, &pCollection);

  CIEHtmlElementCollectionInstance::CreateInstance(&pCollection2);
  if (!pCollection2) {
    pCollection->Release();
    return E_OUTOFMEMORY;
  }
  SibCreateCollectionFromNodeList(pCollection, pCollection2);
  if (pCollection2) {
    pCollection2->AddRef();
  }
  if (pCollection) pCollection->Release();
  *p = pCollection2;

  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_alinkColor(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_alinkColor(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_bgColor(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_bgColor(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_fgColor(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_fgColor(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_linkColor(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_linkColor(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_vlinkColor(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_vlinkColor(VARIANT __RPC_FAR *p) {
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_referrer(BSTR __RPC_FAR *p) {
  nsAutoString value;
  if (m_pNative == NULL || m_pNative->GetReferrer(value)) {
    return E_FAIL;
  }
  *p = SysAllocString(value.GetUnicode());
  return p ? S_OK : E_OUTOFMEMORY;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_location(IHTMLLocation __RPC_FAR *__RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_lastModified(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_URL(BSTR v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_URL(BSTR __RPC_FAR *p) {
  nsAutoString value;
  if (m_pNative == NULL || m_pNative->GetURL(value)) {
    return E_FAIL;
  }
  *p = SysAllocString(value.GetUnicode());
  return p ? S_OK : E_OUTOFMEMORY;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_domain(BSTR v) {
  nsString value(OLE2W(v));
  if (m_pNative == NULL || m_pNative->SetDomain(value)) {
    return E_FAIL;
  }
  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_domain(BSTR __RPC_FAR *p) {
  nsAutoString value;
  if (m_pNative == NULL || m_pNative->GetDomain(value)) {
    return E_FAIL;
  }
  *p = SysAllocString(value.GetUnicode());
  return p ? S_OK : E_OUTOFMEMORY;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_cookie(BSTR v) {
  nsString value(OLE2W(v));
  if (m_pNative == NULL || m_pNative->SetCookie(value)) {
    return E_FAIL;
  }
  return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_cookie(BSTR __RPC_FAR *p) {
  nsAutoString value;
  if (m_pNative == NULL || m_pNative->GetCookie(value)) {
    return E_FAIL;
  }
  *p = SysAllocString(value.GetUnicode());
  return p ? S_OK : E_OUTOFMEMORY;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_expando(VARIANT_BOOL v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_expando(VARIANT_BOOL __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_charset(BSTR v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_charset(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_defaultCharset(BSTR v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_defaultCharset(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_mimeType(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_fileSize(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_fileCreatedDate(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_fileModifiedDate(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_fileUpdatedDate(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_security(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_protocol(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_nameProp(BSTR __RPC_FAR *p)
{
        *p = NULL;
        return E_NOTIMPL;
}


HRESULT CIEHtmlDocument::WriteCommon(SAFEARRAY __RPC_FAR * psarray, int bLn)
{
    HRESULT hr;

    hr = SafeArrayLock(psarray);
    if (FAILED(hr))
    {
        return hr;
    }

    DWORD i = psarray->rgsabound[0].cElements;
    VARIANT *v = (VARIANT *)psarray->pvData;
    VARIANT vStr;

    while (i--)
    {
        if ((hr = VariantChangeType(&vStr, v++, 0, VT_BSTR)))
        {
            SafeArrayUnlock(psarray);
            return hr;
        }

        nsString str(vStr.bstrVal, SysStringLen(vStr.bstrVal));
        if (bLn && !i)
        {
            if (m_pNative->Writeln(str))
            {
                SafeArrayUnlock(psarray);
               return E_FAIL;
            }
        }
        else
        {
            if (m_pNative->Write(str))
            {
                SafeArrayUnlock(psarray);
                return E_FAIL;
            }
        }
    }
    
    hr = SafeArrayUnlock(psarray);
    if (FAILED(hr))
    {
        return hr;
    }

    return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::write(SAFEARRAY __RPC_FAR * psarray)
{
       return WriteCommon(psarray, 0);
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::writeln(SAFEARRAY __RPC_FAR * psarray)
{
      return WriteCommon(psarray, 1);
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::open(BSTR url, VARIANT name, VARIANT features, VARIANT replace, IDispatch __RPC_FAR *__RPC_FAR *pomWindowResult)
{
    if (m_pNative == NULL || m_pNative->Open())
    {
        return E_FAIL;
    }
    return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::close(void)
{
    if (m_pNative == NULL || m_pNative->Close())
    {
        return E_FAIL;
    }
    return S_OK;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::clear(void)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandSupported(BSTR cmdID, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandEnabled(BSTR cmdID, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandState(BSTR cmdID, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandIndeterm(BSTR cmdID, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandText(BSTR cmdID, BSTR __RPC_FAR *pcmdText)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::queryCommandValue(BSTR cmdID, VARIANT __RPC_FAR *pcmdValue)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::execCommand(BSTR cmdID, VARIANT_BOOL showUI, VARIANT value, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::execCommandShowHelp(BSTR cmdID, VARIANT_BOOL __RPC_FAR *pfRet)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::createElement(BSTR eTag, IHTMLElement __RPC_FAR *__RPC_FAR *newElem)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onhelp(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onhelp(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onclick(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onclick(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_ondblclick(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_ondblclick(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onkeyup(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onkeyup(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;

}

HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onkeydown(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onkeydown(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onkeypress(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onkeypress(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onmouseup(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onmouseup(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onmousedown(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onmousedown(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onmousemove(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onmousemove(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onmouseout(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onmouseout(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onmouseover(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onmouseover(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onreadystatechange(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onreadystatechange(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onafterupdate(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onafterupdate(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onrowexit(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onrowexit(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onrowenter(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onrowenter(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_ondragstart(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_ondragstart(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onselectstart(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onselectstart(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::elementFromPoint(long x, long y, IHTMLElement __RPC_FAR *__RPC_FAR *elementHit)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_parentWindow(IHTMLWindow2 __RPC_FAR *__RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_styleSheets(IHTMLStyleSheetsCollection __RPC_FAR *__RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onbeforeupdate(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onbeforeupdate(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::put_onerrorupdate(VARIANT v)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::get_onerrorupdate(VARIANT __RPC_FAR *p)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::toString(BSTR __RPC_FAR *String)
{
        return E_NOTIMPL;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::createStyleSheet(BSTR bstrHref, long lIndex, IHTMLStyleSheet __RPC_FAR *__RPC_FAR *ppnewStyleSheet)
{
        return E_NOTIMPL;
}


///////////////////////////////////////////////////////////////////////////////
// IOleCommandTarget implementation


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::QueryStatus(const GUID __RPC_FAR *pguidCmdGroup, ULONG cCmds, OLECMD __RPC_FAR prgCmds[], OLECMDTEXT __RPC_FAR *pCmdText)
{
        HRESULT hr = E_NOTIMPL;
        if(m_pParent)
        {
                hr = m_pParent->QueryStatus(pguidCmdGroup,cCmds,prgCmds,pCmdText);
        }
        return hr;
}


HRESULT STDMETHODCALLTYPE CIEHtmlDocument::Exec(const GUID __RPC_FAR *pguidCmdGroup, DWORD nCmdID, DWORD nCmdexecopt, VARIANT __RPC_FAR *pvaIn, VARIANT __RPC_FAR *pvaOut)
{
        HRESULT hr = E_NOTIMPL;
        if(m_pParent)
        {
                hr = m_pParent->Exec(pguidCmdGroup,nCmdID,nCmdexecopt,pvaIn,pvaOut);
        }
        return hr;
}


//HRESULT WINAPI func_servProvider(void* pv, REFIID riid, LPVOID* ppv, DWORD dw) {
//  *ppv=NULL;
// InlineIsEqualGUID(,)
//  return E_NOINTERFACE;
//}

HRESULT STDMETHODCALLTYPE CIEHtmlDocument::QueryService(REFGUID guidService,
                                                        REFIID riid,
                                                        void __RPC_FAR *__RPC_FAR *ppv) {
 if (!ppv) return E_UNEXPECTED;
 *ppv=NULL;

 if (IsEqualGUID(guidService,IID_IWebBrowserApp)) {
   if (!m_pParent) {
     //
     // m_pParent is not initialized, when this object
     // is connected to an existing browser instance.
     // In this case, return a dummy web browser.
     //
     if (m_pDummy) {
//       m_pDummy->AddRef();
       return m_pDummy->QueryInterface(IID_IWebBrowserApp, ppv);
     } else {
       return E_UNEXPECTED;
     }
   }
   return m_pParent->QueryInterface(IID_IWebBrowserApp, ppv);
 }

 return E_UNEXPECTED;
}

HRESULT STDMETHODCALLTYPE CIEHtmlDocument::QueryService(REFGUID guidService,
                                                        void __RPC_FAR *__RPC_FAR *ppv) {
 if (!ppv) return E_UNEXPECTED;
//NG_ASSERT(0);

 if (IsEqualGUID(guidService,IID_IWebBrowserApp)) {
   if (!m_pParent) {
     //
     // m_pParent is not initialized, when this object
     // is connected to an existing browser instance.
     // In this case, return a dummy web browser.
     //
     if (m_pDummy) {
//       m_pDummy->AddRef();
       return m_pDummy->QueryInterface(IID_IWebBrowserApp, ppv);
     } else {
       return E_UNEXPECTED;
     }
   }
   return m_pParent->QueryInterface(IID_IWebBrowserApp, ppv);
 }

 *ppv=NULL;

 return E_UNEXPECTED;
}

//
// CConnect class
//
#include "Connect.h"

STDMETHODIMP CMozCtlConnect::Connect(void *pDoc, IDispatch **pDisp) {
  if (!pDisp) return E_INVALIDARG; *pDisp=NULL;
  if (!pDoc) return E_INVALIDARG;

// NG_ASSERT(0);

  CIEHtmlDocumentInstance *pDocInstance = NULL;
  CIEHtmlDocumentInstance::CreateInstance(&pDocInstance);
  if (!pDocInstance) return E_OUTOFMEMORY;
  // addref it so it doesn't go away on us.
  pDocInstance->AddRef();

  CDummyBrowserInstance *pDummyBrowser = NULL;
  CDummyBrowserInstance::CreateInstance(&pDummyBrowser);
  if (!pDummyBrowser) return E_OUTOFMEMORY;
  pDummyBrowser->AddRef();

  nsIDOMDocument *pIDOMDocument = (nsIDOMDocument*)pDoc;

  nsCOMPtr <nsIDOMHTMLDocument> pIDOMHTMLDocument =
    do_QueryInterface(pIDOMDocument);

//  if (!pIDOMDocument) {
//    RETURN_ERROR(_T("get_Document: not HTML"), E_FAIL);
//  }
                
  // give it a pointer to us.  note that we shouldn't be addref'd
  // by this call, or it would be  a circular reference.


  pDocInstance->SetNative(pIDOMHTMLDocument);
  pDocInstance->SetDOMNode(pIDOMDocument);

  pDummyBrowser->pDocInstance = pDocInstance;
  pDocInstance->m_pDummy = pDummyBrowser;

//  pDocInstance->SetParent(NULL);
  return pDocInstance->QueryInterface(IID_IDispatch,(void**)pDisp);
}
