/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */
#include "nsIDOMHTMLMapElement.h"
#include "nsIScriptObjectOwner.h"
#include "nsIDOMEventReceiver.h"
#include "nsIHTMLContent.h"
#include "nsGenericHTMLElement.h"
#include "nsHTMLAtoms.h"
#include "nsHTMLIIDs.h"
#include "nsIStyleContext.h"
#include "nsIMutableStyleContext.h"
#include "nsStyleConsts.h"
#include "nsIPresContext.h"
#include "GenericElementCollection.h"
#include "nsIDocument.h"
#include "nsIHTMLDocument.h"
#include "nsCOMPtr.h"


class nsHTMLMapElement : public nsIDOMHTMLMapElement,
                         public nsIJSScriptObject,
                         public nsIHTMLContent
{
public:
  nsHTMLMapElement(nsINodeInfo *aNodeInfo);
  virtual ~nsHTMLMapElement();

  // nsISupports
  NS_DECL_ISUPPORTS

  // nsIDOMNode
  NS_IMPL_IDOMNODE_USING_GENERIC(mInner)

  // nsIDOMElement
  NS_IMPL_IDOMELEMENT_USING_GENERIC(mInner)

  // nsIDOMHTMLElement
  NS_IMPL_IDOMHTMLELEMENT_USING_GENERIC(mInner)

  // nsIDOMHTMLMapElement
  NS_DECL_IDOMHTMLMAPELEMENT

  // nsIJSScriptObject
  NS_IMPL_IJSSCRIPTOBJECT_USING_GENERIC(mInner)

  // nsIContent
  NS_IMETHOD GetDocument(nsIDocument*& aResult) const {
    return mInner.GetDocument(aResult);                                        
  }                                                                        
  NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep, PRBool aCompileEventHandlers);            
  NS_IMETHOD GetParent(nsIContent*& aResult) const {                       
    return mInner.GetParent(aResult);                                          
  }                                                                        
  NS_IMETHOD SetParent(nsIContent* aParent) {                              
    return mInner.SetParent(aParent);                                          
  }                                                                        
  NS_IMETHOD CanContainChildren(PRBool& aResult) const {                   
    return mInner.CanContainChildren(aResult);                                 
  }                                                                        
  NS_IMETHOD ChildCount(PRInt32& aResult) const {                          
    return mInner.ChildCount(aResult);                                         
  }                                                                        
  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const {         
    return mInner.ChildAt(aIndex, aResult);                                    
  }                                                                        
  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const { 
    return mInner.IndexOf(aPossibleChild, aResult);                            
  }                                                                        
  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex,               
                           PRBool aNotify) {                               
    return mInner.InsertChildAt(aKid, aIndex, aNotify);                        
  }                                                                        
  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,              
                            PRBool aNotify) {                              
    return mInner.ReplaceChildAt(aKid, aIndex, aNotify);                       
  }                                                                        
  NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify) {             
    return mInner.AppendChildTo(aKid, aNotify);                                
  }                                                                        
  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify) {               
    return mInner.RemoveChildAt(aIndex, aNotify);                              
  }                                                                        
  NS_IMETHOD IsSynthetic(PRBool& aResult) {                                
    return mInner.IsSynthetic(aResult);                                        
  }                                                                        
  NS_IMETHOD GetNameSpaceID(PRInt32& aResult) const {                      
    return mInner.GetNameSpaceID(aResult);                                     
  }                                                                        
  NS_IMETHOD GetTag(nsIAtom*& aResult) const {                             
    return mInner.GetTag(aResult);                                             
  }                                                                        
  NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const {
    return mInner.GetNodeInfo(aResult);
  }
  NS_IMETHOD NormalizeAttributeString(const nsAReadableString& aStr,
                                      nsINodeInfo*& aNodeInfo) {
    return mInner.NormalizeAttributeString(aStr, aNodeInfo);
  }                                                                        
  NS_IMETHOD SetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,            
                          const nsAReadableString& aValue, PRBool aNotify) {        
    return mInner.SetAttribute(aNameSpaceID, aName, aValue, aNotify);          
  }                                                                        
  NS_IMETHOD SetAttribute(nsINodeInfo* aNodeInfo,
                          const nsAReadableString& aValue, PRBool aNotify) {
    return mInner.SetAttribute(aNodeInfo, aValue, aNotify);
  }
  NS_IMETHOD GetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,            
                          nsAWritableString& aResult) const {                       
    return mInner.GetAttribute(aNameSpaceID, aName, aResult);                  
  }                                                                        
  NS_IMETHOD GetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,            
                          nsIAtom*& aPrefix, nsAWritableString& aResult) const {
    return mInner.GetAttribute(aNameSpaceID, aName, aPrefix, aResult);
  }
  NS_IMETHOD UnsetAttribute(PRInt32 aNameSpaceID, nsIAtom* aAttribute,     
                            PRBool aNotify) {                              
    return mInner.UnsetAttribute(aNameSpaceID, aAttribute, aNotify);           
  }                                                                        
  NS_IMETHOD GetAttributeNameAt(PRInt32 aIndex,
                                PRInt32& aNameSpaceID,
                                nsIAtom*& aName,
                                nsIAtom*& aPrefix) const {
    return mInner.GetAttributeNameAt(aIndex, aNameSpaceID, aName, aPrefix);
  }
  NS_IMETHOD GetAttributeCount(PRInt32& aResult) const {                   
    return mInner.GetAttributeCount(aResult);                                  
  }                                                                        
  NS_IMETHOD List(FILE* out, PRInt32 aIndent) const {                      
    return mInner.List(out, aIndent);                                          
  }   
  NS_IMETHOD DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const {                      
    return mInner.DumpContent(out, aIndent,aDumpAll);                                          
  } 
  NS_IMETHOD BeginConvertToXIF(nsIXIFConverter* aConverter) const {         
    return mInner.BeginConvertToXIF(aConverter);                               
  }                                                                        
  NS_IMETHOD ConvertContentToXIF(nsIXIFConverter* aConverter) const {       
    return mInner.ConvertContentToXIF(aConverter);                             
  }                                                                        
  NS_IMETHOD FinishConvertToXIF(nsIXIFConverter* aConverter) const {        
    return mInner.FinishConvertToXIF(aConverter);                              
  }                                                                        
  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext,                  
                            nsEvent* aEvent,                               
                            nsIDOMEvent** aDOMEvent,                       
                            PRUint32 aFlags,                               
                            nsEventStatus* aEventStatus);                  

  NS_IMETHOD GetContentID(PRUint32* aID) {
    return mInner.GetContentID(aID);
  }
  NS_IMETHOD SetContentID(PRUint32 aID) {
    return mInner.SetContentID(aID);
  }
  
  NS_IMETHOD RangeAdd(nsIDOMRange& aRange) {                               
    return mInner.RangeAdd(aRange);                                            
  }                                                                        
  NS_IMETHOD RangeRemove(nsIDOMRange& aRange) {                            
    return mInner.RangeRemove(aRange);                                         
  }                                                                        
  NS_IMETHOD GetRangeList(nsVoidArray*& aResult) const {                   
    return mInner.GetRangeList(aResult);                                       
  }      
  
  NS_IMETHOD SetFocus(nsIPresContext* aPresContext) {
    return mInner.SetFocus(aPresContext);
  }

  NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext) {
    return mInner.RemoveFocus(aPresContext);
  }

  NS_IMETHOD GetBindingParent(nsIContent** aContent) {
    return mInner.GetBindingParent(aContent);
  }

  NS_IMETHOD SetBindingParent(nsIContent* aParent) {
    return mInner.SetBindingParent(aParent);
  }  

  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;

  // nsIHTMLContent
  NS_IMPL_IHTMLCONTENT_USING_GENERIC(mInner)

  nsGenericHTMLContainerElement mInner;
  GenericElementCollection* mAreas;
};

